from pyramid.view import view_config
from pyramid.httpexceptions import HTTPFound

from models import db

from datetime import datetime
import os
import simplejson
import pymongo

PROJECT_NAME = "Charm World"
HTTP_CACHE = None

from pyramid.exceptions import NotFound


def _found(res):
    if res is None:
        raise NotFound()
    return res


@view_config(
    route_name="home",
    http_cache=HTTP_CACHE,
    renderer="charmworld:templates/index.pt")
def home(request):
    return {}


@view_config(
    route_name="search",
    renderer="charmworld:templates/search.pt")
def search(request):
    if not "search_text" in request.params:
        url = request.route_url("home")
        return HTTPFound(location=url)
    import xappy
    searcher = xappy.SearchConnection(
        os.path.expanduser("~/projects/charmworld/fts"))
    query = searcher.query_parse(request.params["search_text"])
    import time
    t = time.time()
    results = query.search(0, 30)
    search_time = time.time() - t
    search_time = "%.5f" % search_time
    return {"project": PROJECT_NAME, "results": results,
            "search_time": search_time, "charm_total": searcher.get_doccount()}


@view_config(
    route_name="charm-collection",
    http_cache=HTTP_CACHE,
    renderer="charmworld:templates/charm-collection.pt")
def charm_collection(request):
    charms = _found(db.charms.find(
        {"owner": "charmers"},
        {"name": 1, "owner": 1, "series": 1, "summary": 1, "short_url": 1},
        sort=[("name", pymongo.ASCENDING)]))
    return {"project": PROJECT_NAME, "charms": list(charms)}


@view_config(
    route_name="personal-collection",
    http_cache=HTTP_CACHE,
    renderer="charmworld:templates/charm-collection.pt")
def personal_collection(request):
    charms = _found(db.charms.find(
        {"owner": request.matchdict["owner"]},
        {"name": 1, "owner": 1, "series": 1, "summary": 1, "short_url": 1},
        sort=[("name", pymongo.ASCENDING)]))
    return {"project": PROJECT_NAME, "charms": list(charms)}


@view_config(
    route_name="series-collection",
    http_cache=HTTP_CACHE,
    renderer="charmworld:templates/charm-collection.pt")
def series_collection(request):
    charms = _found(db.charms.find(
        {"owner": "charmers", "series": request.matchdict["series"]},
        {"name": 1, "owner": 1, "series": 1, "summary": 1, "short_url": 1},
        sort=[("name", pymongo.ASCENDING)]))
    return {"project": PROJECT_NAME, "charms": list(charms)}


@view_config(
    route_name="personal-series",
    http_cache=HTTP_CACHE,
    renderer="charmworld:templates/charm-collection.pt")
def personal_series_collection(request):
    charms = _found(db.charms.find(
        {"owner": request.matchdict["owner"],
         "series": request.matchdict["series"]},
        {"name": 1, "owner": 1, "series": 1, "summary": 1, "short_url": 1},
        sort=[("name", pymongo.ASCENDING)]))
    return {"project": PROJECT_NAME, "charms": list(charms)}


@view_config(
    route_name="personal-charm",
    http_cache=HTTP_CACHE,
    renderer="charmworld:templates/charm.pt")
def personal_charm(request):
    charm = _found(db.charms.find_one(
        {"name": request.matchdict['charm'],
         "owner": request.matchdict['owner'],
         "series": request.matchdict['series']}))
    return _charm_view(charm)


@view_config(
    route_name="charm",
    http_cache=HTTP_CACHE,
    renderer="charmworld:templates/charm.pt")
def distro_charm(request):
    charm = _found(db.charms.find_one(
        {"name": request.matchdict["charm"],
         "owner": "charmers",
         "series": request.matchdict["series"]}))
    return _charm_view(charm)


def format_change(change):
    parts = []
    date = datetime.fromtimestamp(change["created"])
    parts.append(date.strftime("%Y/%m/%d"))
    idx = change['committer'].find("<")

    if idx < -1:
        parts.append(change['committer'])
    else:
        parts.append(change['committer'][:idx])

    parts.append(change["message"][:100])
    return " ".join(parts)


def _charm_view(charm):
    if "requires" in charm:
        charm["requires_raw"] = simplejson.dumps(charm["requires"], indent=4)
    if "provides" in charm:
        charm["provides_raw"] = simplejson.dumps(charm["provides"], indent=4)

    charm["d_requires"] = d_requires = []
    for k, v in charm.get("requires", {}).items():
        d = {"name": k, "interface": v["interface"]}
        d_requires.append(d)

    charm["d_provides"] = d_provides = []
    for k, v in charm.get("provides", {}).items():
        d = {"name": k, "interface": v["interface"]}
        d_provides.append(d)

    charm["d_config"] = d_config = []
    for k, v in charm.get("config", {"options": {}})["options"].items():
        d = {"name": k,
             "default": v.get('default', ''),
             'type': v.get('type', ''),
             'description': v.get('description', '')}
        d_config.append(d)

    others = list(db.charms.find(
        {"name": charm["name"], "series": charm["series"],
         "branch_spec": {"$ne": charm["branch_spec"]}}))

    return {"project": PROJECT_NAME, "charm": charm, "name": charm["name"],
            "others": others, 'format_change': format_change}


@view_config(
    route_name="hook",
    renderer="charmworld:templates/hook.pt")
def hook(request):
    charm = _found(db.charms.find_one({
        "owner": "charmers",
        "series": request.matchdict["series"],
        "name": request.matchdict["charm"]}))
    return _hook_view(charm, request.matchdict["hook"])


@view_config(
    route_name="personal-hook",
    renderer="charmworld:templates/hook.pt")
def personal_hook(request):
    charm = _found(db.charms.find_one({
        "owner": request.matchdict["owner"],
        "series": request.matchdict["series"],
        "name": request.matchdict["charm"]}))
    return _hook_view(charm, request.matchdict["hook"])


def _hook_view(charm, hook):
    hook_path = os.path.join(charm["branch_dir"], "hooks", hook)

    with open(hook_path) as fh:
        hook_raw = fh.read()
    return {"project": PROJECT_NAME,
            "hook_raw": hook_raw, "charm": charm, "hook": hook}


@view_config(
    route_name="config",
    http_cache=HTTP_CACHE,
    renderer="charmworld:templates/config.pt")
def config(request):
    charm = _found(db.charms.find_one({
        "owner": "charmers",
        "series": request.matchdict["series"],
        "name": request.matchdict["charm"]}))
    return _config_view(charm)


@view_config(
    route_name="personal-config",
    http_cache=HTTP_CACHE,
    renderer="charmworld:templates/config.pt")
def personal_config(request):
    charm = _found(db.charms.find_one({
        "owner": request.matchdict["owner"],
        "series": request.matchdict["series"],
        "name": request.matchdict["charm"]}))
    return _config_view(charm)


def _config_view(charm):
    return {"charm": charm}


@view_config(
    route_name="interface-collection",
    http_cache=HTTP_CACHE,
    renderer="charmworld:templates/interface-collection.pt")
def interface_collection(request):
    interfaces = {}

    # This is all static between loads
    results = db.charms.find(
        {}, {"name": 1, "i_provides": 1, "i_requires": 1,
             'label': 1, 'short_url': 1})

    for i in results:
        for p in i["i_provides"]:
            if not p in interfaces:
                interfaces[p] = i_info = {
                    "name": p, "provided_by": [], "required_by": []}
            else:
                i_info = interfaces[p]
            i_info["provided_by"].append(i)

        for d in i["i_requires"]:
            if not d in interfaces:
                interfaces[d] = i_info = {
                    "name": d, "provided_by": [], "required_by": []}
            else:
                i_info = interfaces[d]
            i_info["required_by"].append(i)

    # sort them
    interfaces = interfaces.values()
    interfaces.sort(lambda x, y: cmp(x["name"], y["name"]))

    for i in interfaces:
        i["provided_by"] = _unique_sort(i["provided_by"])
        i["required_by"] = _unique_sort(i["required_by"])

    return {"project": PROJECT_NAME, "interfaces": interfaces}


def _unique_sort(charms):

    names = []
    results = []
    for c in charms:
        if c["name"] in names:
            continue
        results.append(c)
        names.append(c["name"])
    results.sort(lambda x, y: cmp(x["name"], y["name"]))
    return results


@view_config(
    route_name="interface", renderer="charmworld:templates/interface.pt")
def interface(request):
    iface = request.matchdict["interface"]
    return {
        "interface": iface,
        "providers": db.charms.find({"i_provides": iface},
                                    sort=[("name", pymongo.ASCENDING)]),
        "requirers": db.charms.find({"i_requires": iface},
                                    sort=[("name", pymongo.ASCENDING)])}
